package graphoutput;

import java.awt.*;
import javax.media.j3d.*;
import javax.vecmath.*;

/**
 * Contains the Shape3D of a 1x1x1-Box, which is transparent and
 * only the edges are visible
 */
public class CoordinateBox extends Object {

   private Appearance app;
   private Shape3D shape3D;

   private static final float[] verts = {
      1.0f,  0.0f,  1.0f,     1.0f,  1.0f,  1.0f,
      0.0f,  1.0f,  1.0f,     0.0f,  0.0f,  1.0f,
      0.0f,  0.0f,  0.0f,     0.0f,  1.0f,  0.0f,
      1.0f,  1.0f,  0.0f,     1.0f,  0.0f,  0.0f,
      1.0f,  1.0f,  1.0f,     1.0f,  1.0f,  0.0f,
      0.0f,  1.0f,  0.0f,     0.0f,  1.0f,  1.0f,
      0.0f,  0.0f,  1.0f,     0.0f,  0.0f,  0.0f,
      1.0f,  0.0f,  0.0f,     1.0f,  0.0f,  1.0f,
	  1.0f,  1.0f,  1.0f,     0.0f,  1.0f,  1.0f,
      1.0f,  0.0f,  1.0f,     0.0f,  0.0f,  1.0f,
      1.0f,  0.0f,  0.0f,     0.0f,  0.0f,  0.0f,
	  1.0f,  1.0f,  0.0f,     0.0f,  1.0f,  0.0f,
   };

   /**
    * Creates the Shape3D of the 1x1x1-box
    */
   public CoordinateBox()
   {
      LineArray lineArray = new LineArray(24, LineArray.COORDINATES);
      lineArray.setCoordinates(0, verts);

      // create the default-appearance (wire-frame)	  
	  app = new Appearance();
	  Color3f color = new Color3f(1.0f, 1.0f, 1.0f);
      ColoringAttributes ca = new ColoringAttributes();
      ca.setColor(color);
      app.setColoringAttributes(ca);	  

      PolygonAttributes pa = new PolygonAttributes();
      pa.setPolygonMode(pa.POLYGON_LINE);
      pa.setCullFace(pa.CULL_NONE);
      app.setPolygonAttributes(pa);

      LineAttributes lat = new LineAttributes();
      lat.setLineWidth(1.0f);
      lat.setLineAntialiasingEnable(true);
      app.setLineAttributes(lat);
	  
      shape3D = new Shape3D(lineArray, app);
   }

   /**
    * Returns the 1x1x1-box as a Shape3D, created by the constructor
    */
   public Shape3D getChild() {
      return shape3D;
   }
}
