

package graphoutput; 

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.picking.*;

 /**
  * Class MarkerEPC
  * Defines a MarkerEPC
  */


class MarkerEPC extends BranchGroup
{
  public static final float HANDLE_SIZE = 0.05f;
  int nr;
  GraphOutput parent;
  public CuttingPlane cp;
  TransformGroup tg; 

  /**
   * Constructor for default-values
   */
  public MarkerEPC (int nr, GraphOutput parent)
  {
    this.nr = nr;
    this.parent = parent;
    // add children
    createComponents(0.5);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the position of the marker is
   *  specified.
   */
  public MarkerEPC(int nr, double pos, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    // add children
    createComponents(pos);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the number and the initial position
   *  are specified
   */
  public MarkerEPC(int nr, double pos, Color3f col, float transparency, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    createComponents(pos, col, transparency);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
  }

  /**
   *  set the position of the MarkerEPC
   */
  public void setPosition(double pos)
  {
     Transform3D t3d = new Transform3D();
     t3d.setTranslation(new Vector3d(1.0, 1.0, pos));
     tg.setTransform(t3d);

     cp.setPosition(new Point3d(0.5, 0.5, pos),
                    new Vector3d(0.0, 0.0, 1.0));

  }


  /**
   *  returns the current position of the cutting-plane
   */
  public double getPoint()
  {

    return 1.0;
  }

  /**
   *  set the color and the transparency of the MarkerEPC
   */
  public void setColor(Color3f col, float transparency)
  {
    cp.setColor(col, transparency);
  }


  /**
   * returns the color of the cuttingplane
   */
  public Color3f getColor()
  {
    return cp.getColor();
  }

  /**
   * returns the transparency of the cuttingplane
   */
  public float getTransparency()
  {
    return cp.getTransparency();
  }

  /**
   * returns a string containing the description of this
   * instance
   */
  public String getDescription()
  {
    return "Nr. "+nr+" at Time "+getTime();
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos with the Color col and the specified transparency
   */
  private void createComponents(double pos, Color3f col, float transparency)
  {
    createComponents(pos);
    setColor(col, transparency);
  }

  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   */
  public double getPosition()
  {
    Transform3D t3d = new Transform3D();
    tg.getTransform(t3d);
    Matrix4d mt = new Matrix4d();
    t3d.get(mt);

    return mt.m23;
  }

  /**
   * returns the current time position of the highlighting marker
   */
  public double getTime()
  {
    Transform3D t3d = new Transform3D();
    tg.getTransform(t3d);
    Matrix4d mt = new Matrix4d();
    t3d.get(mt);
    return parent.time.fitIntoRange(mt.m23);
  }

  /**
   * sets the current time position of the highlighting marker
   */
  public void setTime(double time)
  {
    setPosition(parent.time.normalize(time));
  }


  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   * with max. 3 digits after the dot
   */
  public double getPosition3()
  {
    Integer i = new Integer( (new Double((1000.0*getPosition() ))).intValue() );
	return (i.doubleValue() / 1000.0);
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos
   */
  private void createComponents(double pos)
  {
    Appearance app = new Appearance();
    Color3f objColor = new Color3f(1.0f, 0.5f, 0.0f);

    Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
    app.setMaterial(mat);



    cp = new CuttingPlane(new Point3d(0.0, 0.0, pos),
                          new Vector3d(0.0, 0.0, 1.0));
//    addChild(cp);
    BranchGroup bg = new BranchGroup();
    addChild(bg);
    TransformGroup tg0 = new TransformGroup();
//     Transform3D t = new Transform3D();
//     t.setTranslation(new Vector3d(0.0, 0.0, pos));
//     tg0.setTransform(t);
//     tg0.setCapability(Node.ENABLE_PICK_REPORTING);
//     tg0.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
//     tg0.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    bg.addChild(tg0);
    tg0.addChild(cp);

    tg = new TransformGroup();
     Transform3D t3d = new Transform3D();
     t3d.setTranslation(new Vector3d(1.0, 1.0, pos));

     tg.setTransform(t3d);
     tg.setCapability(Node.ENABLE_PICK_REPORTING);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    tg0.addChild(tg);

    tg.addChild(new Sphere(HANDLE_SIZE/2.0f, app));

    PickTranslateBehaviorEPC mt = new PickTranslateBehaviorEPC(bg,
                                                parent.canvas,
                                                new BoundingSphere(new Point3d(1.0, 1.0, 0.0), (double)HANDLE_SIZE/2.0),
                                                PickObject.USE_BOUNDS, nr,
                                                parent, cp);

    tg.addChild(mt);
  }



} // End of class MarkerEPC