

package graphoutput; 

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.behaviors.picking.*;

 /**
  * Class MarkerLWW
  * Defines a MarkerLWW
  */


class MarkerLWW extends BranchGroup
{
  public static final float HANDLE_SIZE = 0.02f;
  public static final double CIRCLE_RADIUS = 0.1;
  int nr;
  GraphOutput parent;
  public CirclePlane cp;
  TransformGroup tg = new TransformGroup(); 
  TransformGroup tg0 = new TransformGroup();
  double cur_pos;

  /**
   * Constructor for default-values
   */
  public MarkerLWW (int nr, GraphOutput parent)
  {
    this.nr = nr;
    this.parent = parent;
    cur_pos = 0.0;
    // add children
    createComponents(0.0);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the position of the marker is
   *  specified.
   */
  public MarkerLWW(int nr, double pos, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    cur_pos = pos;
    // add children
    createComponents(pos);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
    compile();
  }

  /**
   *  Constructor where the number and the initial position
   *  are specified
   */
  public MarkerLWW(int nr, double pos, Color3f col, float transparency, GraphOutput parent)
  {
    this.nr=nr;
    this.parent = parent;
    cur_pos = pos;
    createComponents(pos, col, transparency);

    setCapability(BranchGroup.ALLOW_DETACH);
    setCapability(BranchGroup.ALLOW_CHILDREN_READ);
    setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
  }

  /**
   *  set the position of the MarkerLWW
   */
  public void setPosition(double pos)
  {
    // pos [0;1]; select the appropr. dataset, and the vector
    // then translate the marker
    cur_pos = pos;
    Point3d p = new Point3d();
    Vector3d v = new Vector3d();
    parent.getLWWOrientation(pos, p, v);
    cp.setPosition(p, v);

    Transform3D t3d = new Transform3D();  
    t3d.rotY(Math.atan2(v.x, v.z));
    t3d.setTranslation(new Vector3d(p.x, p.y, p.z));
    tg0.setTransform(t3d);
    
    Transform3D t3d1 = new Transform3D();
    t3d1.rotZ(Math.PI/4.0);
    tg.setTransform(t3d1);
  }

  /**
   *  redraws the MarkerLWW (if trajectory has changed)
   */
  public void redraw()
  {
    setPosition(cur_pos);
  }



  /**
   *  applies a movement of the MarkerLWW
   */
  public void applyMovement(double delta)
  {
    if(cur_pos+delta >= 0.0 && cur_pos+delta<=1.0)
    {
      cur_pos += delta;
      setPosition(cur_pos);
    }
  }

  /**
   *  returns the current position of the cutting-plane
   */
//  public double getPoint()
//  {
//
//    return cur_pos;
//  }

  /**
   *  set the color and the transparency of the MarkerLWW
   */
  public void setColor(Color3f col, float transparency)
  {
    cp.setColor(col, transparency);
  }


  /**
   * returns the color of the cuttingplane
   */
  public Color3f getColor()
  {
    return cp.getColor();
  }

  /**
   * returns the transparency of the plane
   */
  public float getTransparency()
  {
    return cp.getTransparency();
  }

  /**
   * returns a string containing the description of this
   * instance
   */
  public String getDescription()
  {
    return "Nr. "+nr+" at Time "+getTime();
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos with the Color col and the specified transparency
   */
  private void createComponents(double pos, Color3f col, float transparency)
  {
    createComponents(pos);
    setColor(col, transparency);
  }

  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   */
  public double getPosition()
  {
    return cur_pos;
  }

  /**
   * returns the current time position of the highlighting marker
   */
  public double getTime()
  {
    return parent.time.fitIntoRange(cur_pos);
  }

  /**
   * sets the current time position of the highlighting marker
   */
  public void setTime(double time)
  {
    setPosition(parent.time.normalize(time));
  }


  /**
   * returns the current position of the highlighting marker (between 0 and 1)
   * with max. 3 digits after the dot
   */
  public double getPosition3()
  {
    Integer i = new Integer( (new Double((1000.0*getPosition() ))).intValue() );
	return (i.doubleValue() / 1000.0);
  }

  /**
   * creates the cutting-plane and the sphere at the position
   * pos
   */
  private void createComponents(double pos)
  {
    Appearance app = new Appearance();
    Color3f objColor = new Color3f(1.0f, 0.5f, 0.0f);

    Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
    app.setMaterial(mat);



    cp = new CirclePlane( CIRCLE_RADIUS, 12 );
    setPosition(0.0);
    
//    addChild(cp);
    BranchGroup bg = new BranchGroup();
    addChild(bg);
    tg0 = new TransformGroup();
//     Transform3D t = new Transform3D();
//     t.setTranslation(new Vector3d(0.0, 0.0, pos));
//     tg0.setTransform(t);
//     tg0.setCapability(Node.ENABLE_PICK_REPORTING);
     tg0.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
     tg0.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    bg.addChild(tg0);
    bg.addChild(cp);


//     tg.setCapability(Node.ENABLE_PICK_REPORTING);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
     tg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    tg0.addChild(tg);

    TransformGroup htg = new TransformGroup();
    tg.addChild(htg);
     Transform3D t0 = new Transform3D();
     htg.setCapability(Node.ENABLE_PICK_REPORTING);
     htg.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
     htg.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
     t0.setTranslation(new Vector3d(0.0, CIRCLE_RADIUS, 0.0));
     htg.setTransform(t0);

    BranchGroup bg1 = new BranchGroup();
    bg1.addChild(new Sphere(HANDLE_SIZE/2.0f, app));
    bg1.setCapability(BranchGroup.ENABLE_PICK_REPORTING);
    htg.addChild(bg1);

    PickTranslateBehaviorLWW mt = new PickTranslateBehaviorLWW(bg1,
                                                parent.canvas,
                                                new BoundingSphere(new Point3d(0.0, CIRCLE_RADIUS, 0.0), (double)HANDLE_SIZE/2.0),
                                                PickObject.USE_BOUNDS, nr,
                                                parent, this);

    htg.addChild(mt);
  }



} // End of class MarkerLWW