package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.behaviors.picking.*;

/**
 * MouseTranslate2 is a Java3D behavior object that lets users control the
 * translation (X, Y) of an object via a mouse drag motion with the third
 * mouse button (alt-click on PC). See MouseRotate for similar usage info.
 */

public class MouseTranslateTDC extends MouseBehavior {
    double x_factor = .02;
    double y_factor = .02;
    Vector3d translation = new Vector3d();
    int nr;
    GraphOutput go;
    MarkerTDC marker;

  private MouseBehaviorCallback callback = null;

  /**
   * Creates a mouse translate behavior given the transform group.
   * @param transformGroup The transformGroup to operate on.
   * @param xpos the fixed x-value of the transformation (z is always 0.0)
   */
  public MouseTranslateTDC(TransformGroup transformGroup, int nr, GraphOutput go, MarkerTDC marker) {
    super(transformGroup);
    this.nr=nr;   
    this.go=go;
    this.marker=marker;
  }

  /**
   * Creates a default translate behavior.
   * @param xpos the fixed x-value of the transformation (z is always 0.0)
   */
  public MouseTranslateTDC(int nr, GraphOutput go, MarkerTDC marker){
    super(0);
    this.nr=nr;
    this.go=go;
    this.marker=marker;
  }

  /**
   * Creates a translate behavior.
   * Note that this behavior still needs a transform
   * group to work on (use setTransformGroup(tg)) and
   * the transform group must add this behavior.
   * @param flags
   */
  public MouseTranslateTDC(int flags, int nr, GraphOutput go, MarkerTDC marker) {
    super(flags);
    this.nr=nr;
    this.go=go;
    this.marker=marker;
  }

  public void initialize() {
    super.initialize();
    if ((flags & INVERT_INPUT) == INVERT_INPUT) {
       invert = true;
       x_factor *= -1;
       y_factor *= -1;
    }
  }

  /**
   * Return the x-axis movement multipler.
   **/

  public double getXFactor() {
    return x_factor;
  }

  /**
   * Return the y-axis movement multipler.
   **/

  public double getYFactor() {
    return y_factor;
  }

  /**
   * Set the x-axis amd y-axis movement multipler with factor.
   **/

  public void setFactor( double factor) {
    x_factor = y_factor = factor;

  }

  /**
   * Set the x-axis amd y-axis movement multipler with xFactor and yFactor
   * respectively.
   **/

  public void setFactor( double xFactor, double yFactor) {
    x_factor = xFactor;
    y_factor = yFactor;
  }

  public void processStimulus (Enumeration criteria) {
    WakeupCriterion wakeup;
    AWTEvent[] event;
    int id;
    int dx, dy;

    while (criteria.hasMoreElements()) {
      wakeup = (WakeupCriterion) criteria.nextElement();

      if (wakeup instanceof WakeupOnAWTEvent) {
	event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
	for (int i=0; i<event.length; i++) {
	  processMouseEvent((MouseEvent) event[i]);

	  if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
	      ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0))){
	    id = event[i].getID();
	    if ((id == MouseEvent.MOUSE_DRAGGED) &&
		!((MouseEvent)event[i]).isAltDown() &&
		((MouseEvent)event[i]).isMetaDown()){


	      x = ((MouseEvent)event[i]).getX();
	      y = ((MouseEvent)event[i]).getY();

	      dx = x - x_last;
	      dy = y - y_last;

	      if ((!reset) && ((Math.abs(dy) < 50) && (Math.abs(dx) < 50))){
		//System.out.println("dx " + dx + " dy " + dy);
		transformGroup.getTransform(currXform);

		translation.x = -dx*x_factor;
//		translation.y = -dy*y_factor;

		transformX.set(translation);

		if (invert) {
		  currXform.mul(currXform, transformX);
		} else {
		  currXform.mul(transformX, currXform);
		}

        marker.applyMovement(translation.x);
        go.setHighlightInfo(nr, marker.getPosition());
        
		transformChanged( currXform );

		if (callback!=null)
		    callback.transformChanged( MouseBehaviorCallback.TRANSLATE,
					       currXform );

	      }
	      else {
		reset = false;
	      }
	      x_last = x;
	      y_last = y;
	    }
	    else if (id == MouseEvent.MOUSE_PRESSED) {
	      x_last = ((MouseEvent)event[i]).getX();
	      y_last = ((MouseEvent)event[i]).getY();
	    }
	  }
	}
      }
    }

    wakeupOn (mouseCriterion);
  }

  /**
    * Users can overload this method  which is called every time
    * the Behavior updates the transform
    *
    * Default implementation does nothing
    */
  public void transformChanged( Transform3D transform ) {
  }

  /**
    * The transformChanged method in the callback class will
    * be called every time the transform is updated
    */
  public void setupCallback( MouseBehaviorCallback callback ) {
      this.callback = callback;
  }
}

