package graphoutput;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import javax.media.j3d.*;
import javax.vecmath.*;
 /**
  * Class Style
  * Defines the style of an output
  */

class Style
//implements Serializable
{
  public static final double DEFAULT_MAX_WING_LENGTH = 0.1;
  public static final float  DEFAULT_WING_TRANSPARENCY = 0.8f;

  public Vector       marker_v;
  public int          ord;
  public Vector       cuttingplane_v;
  public Vector       color_v;
  public double       transparency;
  public boolean      showgrid;
  public boolean      showscaling;

  public Vector       app_v;

  // --------- for EPC and TDC -------------
  public boolean      showlines;

  // ------------- for EPC -----------------
  public double       space_frac;

  // ------------- for LWW -----------------
  public double       maxwinglength;
  public float        wingtransparency;

  // ------------- for TDC -----------------
  public boolean      showsurface;
  public boolean      showplanes;
  public float        sur_transparency;

  /**
   * Constructor
   * for initial values (transparency=0.0, showgrid=false)
   */
  public Style ()
  {
    marker_v      = new Vector();
    cuttingplane_v = new Vector();
    color_v        = new Vector();
    transparency   = 0.0;
    showgrid       = false;
	showscaling    = false;
	space_frac     = 0.7;
	showlines      = false;
	app_v          = new Vector();
    ord            = 0;
    maxwinglength  = DEFAULT_MAX_WING_LENGTH;
    wingtransparency = DEFAULT_WING_TRANSPARENCY;
    showsurface    = true;
    showplanes     = true;
    sur_transparency = 0.6f;

    // create 16 appearances with different colors
    for (int i=0; i<16; i++)
    {
  	  Appearance app = new Appearance();
      Color3f objColor = new Color3f((float)ColorDlg.COLOR[i].getRed()/255.0f,
                                     (float)ColorDlg.COLOR[i].getGreen()/255.0f,
                                     (float)ColorDlg.COLOR[i].getBlue()/255.0f);
      Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
      mat.setCapability(Material.ALLOW_COMPONENT_READ);
      mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
      app.setMaterial(mat);
      app.setCapability(Appearance.ALLOW_MATERIAL_READ);
      app.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
      PolygonAttributes pa = new PolygonAttributes();
      pa.setCullFace(pa.CULL_NONE);
      app.setPolygonAttributes(pa);

	  app_v.addElement(app);
    }

  }

  /**
   * Constructor
   */
  public Style (double transparency, boolean showgrid, boolean showscaling, double space_frac, boolean showlines)
  {
    this.transparency = transparency;
	this.showgrid     = showgrid;
	this.showscaling  = showscaling;
    this.space_frac   = space_frac;
	this.showlines    = showlines;
    marker_v      = new Vector();
    cuttingplane_v    = new Vector();
    color_v           = new Vector();
	app_v             = new Vector();
    ord               = 0;
    maxwinglength     = DEFAULT_MAX_WING_LENGTH;
    wingtransparency = DEFAULT_WING_TRANSPARENCY;
    showsurface     = true;
    showplanes     = true;
    sur_transparency = 0.6f;

    // create 16 appearances with different colors
    for (int i=0; i<16; i++)
    {
  	  Appearance app = new Appearance();
      Color3f objColor = new Color3f((float)ColorDlg.COLOR[i].getRed()/255.0f,
                                     (float)ColorDlg.COLOR[i].getGreen()/255.0f,
                                     (float)ColorDlg.COLOR[i].getBlue()/255.0f);
      Material mat = new Material(objColor, new Color3f(0.0f,0.0f,0.0f), objColor,
	                                              new Color3f(1.0f,1.0f,1.0f), 80.0f);
      mat.setCapability(Material.ALLOW_COMPONENT_READ);
      mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
      app.setMaterial(mat);
      app.setCapability(Appearance.ALLOW_MATERIAL_READ);
      app.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
      PolygonAttributes pa = new PolygonAttributes();
      pa.setCullFace(pa.CULL_NONE);
      app.setPolygonAttributes(pa);

	  app_v.addElement(app);
    }

  }

  /**
   * adds one CuttingPlane to the Vector and
   * returns the index of this plane
   */
  public int addCuttingPlane(BranchGroup bg, CuttingPlane cp)
  {
//    bg.addChild(cp);

    cuttingplane_v.addElement( cp );
	return (cuttingplane_v.size()-1);
  }

  /**
   * inserts the whole vector of cuttingplanes to the
   * specified branchgroup
   */
  public void redrawCuttingPlanes(BranchGroup bg)
  {
    for (int i=0; i<cuttingplane_v.size(); i++)
    {
      bg.addChild((CuttingPlane)cuttingplane_v.elementAt(i));
    }

  }

  /**
   * inserts the whole vector of markers to the
   * specified branchgroup
   */
  public void redrawMarkers(BranchGroup bg)
  {
    for (int i=0; i<marker_v.size(); i++)
    {
      if(marker_v.elementAt(i) instanceof MarkerEPC)
        bg.addChild((MarkerEPC)marker_v.elementAt(i));
      else if(marker_v.elementAt(i) instanceof MarkerLWW)
      {
        bg.addChild((MarkerLWW)marker_v.elementAt(i));
        ((MarkerLWW)marker_v.elementAt(i)).redraw();
      }
      else if(marker_v.elementAt(i) instanceof MarkerTDC)
      {
        bg.addChild((MarkerTDC)marker_v.elementAt(i));
        ((MarkerTDC)marker_v.elementAt(i)).redraw();
      }
//extend
    }

  }

  /**
   * repositions the markers if the trajectory has changed
   */
  public void resetMarkers()
  {
    for (int i=0; i<marker_v.size(); i++)
    {
      if(marker_v.elementAt(i) instanceof MarkerLWW)
      {
        ((MarkerLWW)marker_v.elementAt(i)).redraw();
      }
      else if(marker_v.elementAt(i) instanceof MarkerTDC)
      {
        ((MarkerTDC)marker_v.elementAt(i)).redraw();
      }
//extend
    }
  }

  /**
   * return an upwards increasing unique number
   * starting at 1
   */
  public int getUniqueNumber()
  {
    ord++;
    return ord;
  }


  /**
   * deletes the CuttingPlane with the specified index
   */
  public void delCuttingPlane(int index)
  {
    cuttingplane_v.removeElementAt(index);
  }

  /**
   * return the vector of the CuttingPlanes
   */
  public Vector getCuttingPlanes()
  {
    return cuttingplane_v;
  }

  /**
   * sets the current color-vector
   */
  public void setColors(Vector newcolors)
  {
    color_v = (Vector)newcolors.clone();
  }

  /**
   * gets the current color-vector
   */
  public Vector getColors()
  {
    return color_v;
  }

  /**
   * adds the specified appearance to the
   * appearance-vector
   */
  public void addAppearance(Appearance app)
  {
    app.setCapability(Appearance.ALLOW_MATERIAL_READ);
    app.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
    app_v.addElement(app);
  }

  /**
   * empties the appearance-vector and then adds
   * the specified appearance
   */
  public void initAppearance(Appearance app)
  {
    app_v = new Vector();
    app.setCapability(Appearance.ALLOW_MATERIAL_READ);
    app.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
    app_v.addElement(app);
  }

  /**
   * gets the indexed appearance. if the index does
   * not exist, the appearance of index 0 is returned
   */
  public Appearance getAppearance(int index)
  {
    Appearance app;

    if(app_v.size()<index+1) app = (Appearance)app_v.elementAt(0);
     else app = (Appearance)app_v.elementAt(index);

    return app;
  }     
   



}

